# @Coding: UTF-8
# @Time: 2025/3/28 23:50
# @Author: xieyang_ls
# @Filename: thread_holder.py

from threading import current_thread

from typing import TypeVar, Generic, Iterator

from pyutils_spirit.util.assemble import Assemble, HashAssemble

K = TypeVar('K')

V = TypeVar('V')


class ThreadHolder(Generic[K, V]):

    def __init__(self):
        self.__holders: Assemble[int | K, V] = HashAssemble()

    def __len__(self) -> int:
        return len(self.__holders)

    def __iter__(self) -> Iterator[int | K]:
        return iter(self.__holders)

    def __next__(self) -> int | K:
        return self.__holders.__next__()

    def set_thread_holder(self, instance: V) -> None:
        current_thread_id: int = current_thread().ident
        self.__holders[current_thread_id] = instance

    def get_thread_holder(self) -> V:
        current_thread_id = current_thread().ident
        return self.__holders[current_thread_id]

    def remove_thread_holder(self) -> V:
        current_thread_id = current_thread().ident
        return self.__holders.remove(key=current_thread_id)

    def set_holder(self, key: K, value: V) -> None:
        self.__holders[key] = value

    def get_holder(self, key: K) -> V:
        return self.__holders[key]

    def remove_holder(self, key: K) -> V:
        return self.__holders.remove(key=key)



# @Coding: UTF-8
# @Time: 2024/9/22 17:13
# @Author: xieyang_ls
# @Filename: __init__.py.py

from pyutils_spirit.util.cities import get_all_cities

from pyutils_spirit.spirit_container.annotation import (component,
                                                        mapper,
                                                        service,
                                                        controller,
                                                        resource,
                                                        get, post, put, delete,
                                                        exception_advice,
                                                        throws_exception,
                                                        request_interceptor,
                                                        interceptor_before,
                                                        interceptor_after)

from pyutils_spirit.spirit_container.multipart_file import MultipartFile

from pyutils_spirit.spirit_container.request_result import Result

from pyutils_spirit.spirit_container.spirit_application import SpiritApplication

from pyutils_spirit.spirit_container.spirit_application_container import SpiritApplicationContainer

__all__ = ["component",
           "mapper",
           "service",
           "controller",
           "resource",
           "get",
           "post",
           "put",
           "delete",
           "exception_advice",
           "throws_exception",
           "request_interceptor",
           "interceptor_before",
           "interceptor_after",
           "MultipartFile",
           "Result",
           "SpiritApplication",
           "SpiritApplicationContainer"]


@component(signature="cities_component")
class CitiesComponent:
    def __init__(self):
        self.get_all_cities: callable = get_all_cities



from typing import Any

from pyutils_spirit.util.assemble import Assemble, HashAssemble

from pyutils_spirit.exception.exception import ArgumentException


class SpiritApplicationContainer:
    __instance: object = None

    __application_container: Assemble[str, Any] = None

    def __init__(self):
        if self.__application_container is None:
            self.__application_container = HashAssemble()

    def __new__(cls) -> object:
        if cls.__instance is None:
            cls.__instance = object.__new__(cls)
        return cls.__instance

    def set_resource(self, signature: str, resource: Any):
        if signature in self.__application_container:
            raise ArgumentException(f"spirit_application_container: '{signature}' "
                                    f"Different resources cannot have the same signature")
        self.__application_container[signature] = resource

    def get_resource(self, signature: str) -> Any:
        return self.__application_container[signature]

    def remove_resource(self, signature: str) -> Any:
        return self.__application_container.remove(key=signature)

    def get_capacity(self):
        return len(self.__application_container)

    def __iter__(self):
        return iter(self.__application_container)

    def __next__(self):
        return next(self.__application_container)
